"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var inquirer = require("inquirer");
var _ = require("lodash");
var open = require("open");
var manifest_1 = require("../util/manifest");
var login_1 = require("../util/login");
var glob = require("glob");
var fs = require("fs-extra");
var log_1 = require("../util/log");
var appStructure = require("../util/appStructure");
var models_1 = require("../models");
var multichoice_1 = require("../util/multichoice");
module.exports = function (program) {
    program
        .command('publish')
        .description('publish a new or existing Custom App design')
        .option('-g, --go', 'navigate to design in Asset Library after publishing')
        .option('-m, --manifest <value>', 'use a specific manifest file')
        .action(function (options) {
        var manifest = manifest_1.Manifest.get(options.manifest);
        login_1.Login.getMostRecentLogin()
            .then(function (domo) {
            if (domo.getRefreshToken() === undefined) {
                log_1.log.notAuthenticated();
            }
            checkFileNames(manifest);
            if (!manifest.id) {
                createDesign(domo, manifest, options);
            }
            else {
                domo
                    .getDesign(manifest.id)
                    .then(function (result) {
                    checkMapping(result, manifest);
                    return publishAssets(domo, manifest).then(function () {
                        return linkToDomoWeb(options, domo, manifest);
                    });
                })
                    .catch(function (e) { return publishErrors(e, domo, manifest); });
            }
        })
            .catch(log_1.log.notAuthenticated);
    });
};
function createDesign(domo, manifest, options) {
    return (domo
        .createDesign(manifest)
        //@ts-ignore
        .then(function (designResult) {
        var design = designResult[0];
        // add the version returned when design published
        design.version = designResult[1].version;
        return manifest_1.Manifest.persistDesignId(design, manifest);
    })
        //@ts-ignore
        .tap(function () {
        log_1.log.ok('New design created on ' + domo.getInstance());
    })
        //@ts-ignore
        .catch(function (e) {
        log_1.log.fail('Error when creating new design:', e.message);
    })
        .then(domo.uploadAllAssets.bind(domo, manifest))
        .then(logUploadSuccess)
        .catch(function (e) {
        log_1.log.fail(e || 'Error uploading assets in uploading assets for a new design');
    })
        .then(function () {
        linkToDomoWeb(options, domo, manifest);
    }));
}
function logUploadSuccess(files) {
    files.forEach(function (file) {
        log_1.log.ok('Uploaded: ' + file.path);
    });
}
function linkToDomoWeb(options, domo, manifest) {
    console.log('Design can be found at https://' +
        domo.getInstance() +
        '/assetlibrary?designId=' +
        manifest.id);
    if (options && options['go']) {
        open('https://' + domo.getInstance() + '/assetlibrary?designId=' + manifest.id);
    }
}
function checkMapping(result, manifest) {
    var latestVersionNumber = result.latestVersion;
    var latestVersion = result.versions.filter(function (obj) {
        return obj.version === latestVersionNumber;
    });
    var oldMapping = latestVersion[0].mapping[0];
    var newMapping = manifest.mapping[0];
    if (oldMapping) {
        delete oldMapping.dql;
    }
    if (!_.isEqual(oldMapping, newMapping)) {
        log_1.log.warn('Mapping has changed. Check current cards for potential miss-mappings.');
    }
}
function checkFileNames(manifest) {
    var ignore = ['**/*/node_modules/**/*', 'node_modules/**/*'];
    var userIgnore = manifest.ignore;
    if (userIgnore) {
        if (userIgnore instanceof Array) {
            ignore = ignore.concat(userIgnore);
        }
        else {
            log_1.log.fail('The manifest.json "ignore" property must be an Array');
        }
    }
    var badFileList = glob.sync('**/*', { ignore: ignore }).filter(function (file) {
        console.log(file);
        return (
        // special characters except "/" in file names are not allowed
        /[~`!#$%\^&*+=\[\]\\';,{}|\\":<>\?]/g.test(file) ||
            (!fs.lstatSync(file).isDirectory() && file.indexOf(' ') !== -1));
    });
    if (badFileList.length > 0) {
        log_1.log.fail('Special characters in file/directory names are not allowed as well as spaces in file names. Please rename the following file/directory(s): ' +
            badFileList);
    }
    if (!appStructure.hasThumbnail()) {
        log_1.log.warn(models_1.constant.THUMBNAIL_CREATE_WARNING, models_1.constant.CREATE_THUMBNAIL);
    }
}
function recreateDesign(domo, manifest) {
    inquirer
        .prompt((0, multichoice_1.default)({
        type: 'list',
        message: 'This design has been deleted. Would you like to undelete this design and publish to it, or create a new design with these assets?',
        name: 'askRecreate',
        choices: ['Undelete', 'Create New Design'],
    }))
        .then(function (answers) {
        if (answers.askRecreate) {
            if (answers.askRecreate == 'Create New Design') {
                manifest.id = null;
                createDesign(domo, manifest);
            }
            else if (answers.askRecreate == 'Undelete') {
                domo.unDeleteDesign(manifest.id).then(function () {
                    publishAssets(domo, manifest);
                });
            }
            else {
                log_1.log.ok('Exited without republishing design');
            }
        }
    });
}
function publishErrors(e, domo, manifest) {
    if (e.statusCode === 400 && !e.message.startsWith('DA')) {
        log_1.log.fail('Design failed to publish with error:', e.message);
    }
    else if (e.statusCode === 401) {
        log_1.log.notAuthenticated(domo.getRefreshToken());
    }
    else if (e.statusCode === 403) {
        log_1.log.fail('You do not have access to the design with id ' +
            manifest.id +
            ' on ' +
            domo.getInstance() +
            '.\nIf that seems unlikely, other possibilities are that you are publishing this design to a new environment other than the one in which it was first created.' +
            '\nAlternatively, you may have inadvertently changed the design id.');
    }
    else if (e.message.substr(0, 23) == 'Design has been deleted')
        recreateDesign(domo, manifest);
    else if (e.message.substr(0, 15) == 'No design found') {
        inquirer
            .prompt({
            type: 'confirm',
            message: 'This design does not exist on ' +
                domo.getInstance() +
                '. Would you like to create a new design with these assets?',
            name: 'askCreate',
        })
            .then(function (answers) {
            if (answers.askCreate) {
                manifest.id = null;
                createDesign(domo, manifest);
            }
        });
    }
    else if (e.message && e.message.startsWith('DA')) {
        switch (e.message.substr(0, 6)) {
            case 'DA0086':
                log_1.log.warn(e.message);
                inquirer
                    .prompt([
                    {
                        type: 'confirm',
                        message: 'This design has been deleted. Would you like to publish a new design with these assets?',
                        name: 'askRecreate',
                    },
                ])
                    .then(function (answers) {
                    if (answers.askRecreate) {
                        manifest.id = null;
                        createDesign(domo, manifest);
                    }
                });
                break;
            default:
                // Pass on error messages that aren't in the set of documented errors.
                log_1.log.fail(e.message);
                break;
        }
    }
    else {
        //Error uploading assets
        log_1.log.fail(log_1.log.handleErrorMessage(e, 'Error uploading assets in the upload all assets'));
    }
}
function publishAssets(domo, manifest) {
    return (domo
        .uploadAllAssets(manifest)
        //@ts-ignore
        .tap(function () {
        log_1.log.ok('Publishing ' +
            manifest.name +
            ' to ' +
            domo.getInstance() +
            ' on ' +
            new Date());
    })
        .then(logUploadSuccess)
        //@ts-ignore
        .catch(function (e) {
        publishErrors(e, domo, manifest);
    }));
}
//# sourceMappingURL=publish.js.map